#!/usr/bin/perl -w
# Change above so it points to perl.


########
#
# Content Management Script, editor section, (c) Alex Cruickshank 2002 (alex@itreviews.co.uk)
#
# This code is released under the GNU Public Licence (see www.gnu.org for details) and is supplied
# as is with no warranty either expressed or implied. No liability accepted for loss or damage of any kind.
#
# Always back up your data before using new software.
#
# You are free to use this program to manage and serve data on your site.
# Please notify the author of any such use, as well as bug reports, modifications, etc.
#
#
# the URL of this file
$script_name = "editor.cgi";
#
# the URL of the image upload script
$upload_script = "upload.pl";
#
# the URL of the image upload script
$content_script = "content.cgi";
#
# title for editor pages
$title = "PC Magazine - content generator";
#
# file path to images (not URL), with trailing '/'
$image_path = "../images/";
#
# URL to images, as above, with trailing '/'
$image_url = "/images/";
#
# file path to content text files, with trailing '/'
$content_path = "../data/";
#
########


# next line is required for calculating the size of embedded images
use Image::Size;

# apply some basic security and error reporting rules
&init_security;

# prepare the browser for HTML content
print "Content-type: text/html\n\n";

# handle the POSTed form data
&parse_form;

# decide what to do
if ($FORM{'data_to_process'}) {
    if ($FORM{'data_to_process'} eq "get_input") {
        &get_input;
    }
    elsif ($FORM{'data_to_process'} eq "create_new_file") {
        &empty_vars;
        &show_input_form;
    }
    elsif ($FORM{'data_to_process'} eq "edit_existing_file") {
        &empty_vars;
        &load_file_data;
        &show_input_form;
    }
}
elsif ($FORM{'show_help'}) {
    &show_help;
}
else {
    &show_initial_choice;
}

exit;


########


sub show_initial_choice {
    # give the user the choice of creating a new file or editing an existing one

    &header;

    # find out what files are available, ready to supply as SELECT list
    opendir(ROOT,$content_path);
    @files_temp = readdir(ROOT);
    closedir(ROOT);
    @files = sort {$a cmp $b} @files_temp;

    print <<"OUTPUT";
    <P>Please choose one of the following options.</P>

    <form method="POST" action="$script_name">
    <input type="hidden" name="data_to_process" value="create_new_file">
    <input type="submit" name="Enter" value="Create a new page">
    </form>
OUTPUT

    # check if there are any files present before offering the edit option
    $number_of_files=0;
    foreach (@files) {
        if ($_ =~ m/\.txt$/i) {$number_of_files++;}
    }

    unless ($number_of_files < 1) {

        print <<"OUTPUT";
        <form method="POST" action="$script_name">
        <input type="hidden" name="data_to_process" value="edit_existing_file">
        <input type="submit" name="Enter" value="Load and edit"> the following file: 
        <SELECT NAME="content_file">
        <OPTION VALUE="" SELECTED> (choose)
OUTPUT

        foreach (@files) {
            if ($_ !~ m/\.txt$/i) {next;}
            print "<OPTION VALUE=\"$_\"> $_";
        }

        print <<"OUTPUT";
        </SELECT>
        </form>
OUTPUT
    }

    &footer;
    exit;
}


sub empty_vars {
    # make sure that all variables are empty before either showing blank form or loading data from file

    $heading="";
    $pagename="";
    $description="";
    $keywords="";
    $publication_date="";
    $body="";
    $note="";
}


sub load_file_data {
    # load the contents of a file ready for editing

    # check the file exists, load its contents if so
    $content_file = $content_path . ($FORM{'content_file'});

    unless (($FORM{'content_file'})&&(-e "$content_file")) {
        &header;
        print "<P>File not found - please go back and try again.</P>\n";
        &footer;
        exit;
    }

    open (CONTENTS, "$content_file");
    @content_file = <CONTENTS>;
    close (CONTENTS);

    foreach $readline (@content_file) {
        if ($readline =~ m/^heading\=(.*)/i) {chomp($heading=$1);}
        elsif ($readline =~ m/^pagename\=(.*)/i) {chomp($filepagename=$1);}
        elsif ($readline =~ m/^description\=(.*)/i) {chomp($description=$1);}
        elsif ($readline =~ m/^keywords\=(.*)/i) {chomp($keywords=$1);}
        elsif ($readline =~ m/^publication_date\=(.*)/i) {chomp($publication_date=$1);}
        elsif ($readline =~ m/^body\=(.*)/i) {chomp($body=$1);}
        elsif ($readline =~ m/^note\=(.*)/i) {chomp($note=$1);}
    }

    # check that the pagename matches
    $pagename = ($FORM{'content_file'});
    $pagename =~ s/\.txt$//i;
    unless ($filepagename eq $pagename) {
        &header;
        print "<P>Problem - filenames don't match. Please check contents of $pagename.txt manually.</P>\n";
        &footer;
        exit;
    }

    # 'unconvert' embedded proprietary codes to HTML format - not relevant for pagename, keywords and publication date
    $heading = &unprocess_html_tags($heading);
    $description = &unprocess_html_tags($description);
    $body = &unprocess_html_tags($body);
    $note = &unprocess_html_tags($note);
}


sub show_input_form {
    # send the input form to the browser

    &header;

    # only print the link to the 'upload images' script if it actually exists
    if (-e "$upload_script") {

        print <<"OUTPUT";    
        <FORM target="_BLANK" action="$upload_script">
        <input type="submit" value="Click to upload images">
        </FORM>
OUTPUT
    }

    print <<"OUTPUT";

    <FORM target="_BLANK" method="POST" action="$script_name">
    <input type="submit" name="show_help" value="Click here for help">
    </FORM>

    <form method="POST" action="$script_name">
    <input type="hidden" name="data_to_process" value="get_input">

    <P>Heading (in full):<BR>
    <input type="text" size="25" name="heading" value="$heading"></P>

    <P>Pagename (no trailing '.htm' or other extension):<BR>
    <input type="text" size="10" name="pagename" value="$pagename"></P>

    <P>Description (short and snappy, will be shown under title and used in meta tags):<BR>
    <input type="text" size="35" name="description" value="$description"></P>

    <P>Keywords (purely for search engine use):<BR>
    <input type="text" size="55" name="keywords" value="$keywords"></P>

    <P>Publication date (dd/mm/yyyy format, numeric):<BR>
    <input type="text" size="15" name="publication_date" value="$publication_date"></P>

    <P>Body text:<BR>
    <textarea name="body" rows="20" cols="80" WRAP="VIRTUAL">$body</textarea></P>

    <P>Note text (if any, as above):<BR>
    <textarea name="note" rows="2" cols="80" WRAP="VIRTUAL">$note</textarea></P>

    <input type="submit" name="Enter" value="Generate page">
    </FORM>

OUTPUT

    &footer;
}


sub get_input {
    # take the form values and put them into variables

    $heading=$FORM{'heading'};
    $pagename=lc($FORM{'pagename'});
    $description=$FORM{'description'};
    $keywords=$FORM{'keywords'};
    $publication_date=$FORM{'publication_date'};
    $body=$FORM{'body'};
    $note=$FORM{'note'};

    # do some error checking on the input variables
    $input_error = "";

    unless (($heading)&&($pagename)&&($description)&&($keywords)&&($body)) {$input_error .= "Missing data - not all fields were completed.<BR>\n";}
    if (length($heading) > 50) {$input_error .= "Heading too long.<BR>\n";}
    if (length($pagename) > 15) {$input_error .= "Pagename too long.<BR>\n";}
    if ($pagename =~ /\W/) {$input_error .= "Pagename must only contain alphanumeric characters.<BR>\n";}
    if (length($description) > 50) {$input_error .= "Description too long.<BR>\n";}
    if (length($keywords) > 120) {$input_error .= "Keyword list too long.<BR>\n";}

    if ($publication_date =~ /(\d{2})\/(\d{2})\/(\d{4})/) {
        unless(datecheck(eval($3), eval($2), eval($1))) {$input_error .= "Publication date format incorrect.<BR>\n";}
    }
    else {$input_error .= "Publication date format incorrect.<BR>\n";}

    # send the results to the browser
    if ($input_error) {
        &header;
        print "<P>There were problems with the data you submitted. Please go back and try again. The error(s) are listed below;</P><P>$input_error</P>\n";
        &footer;
        exit;
    }

    # convert embedded HTML codes to proprietary format - not relevant for pagename, keywords and publication date
    $heading = &process_html_tags($heading);
    $description = &process_html_tags($description);
    $body = &process_html_tags($body);
    $note = &process_html_tags($note);

    # find the dimensions of any embedded images (must have been uploaded first)
    &dimension_body_images;

    # now write the contents to a file, over-writing any existing file
    $full_textfilename = $content_path . $pagename . ".txt";
    open (TARGET, ">$full_textfilename");
    print TARGET "heading=$heading\n";
    print TARGET "pagename=$pagename\n";
    print TARGET "description=$description\n";
    print TARGET "keywords=$keywords\n";
    print TARGET "publication_date=$publication_date\n";
    print TARGET "body=$body\n";
    print TARGET "note=$note\n";
    close (TARGET);

    # say we've finished
    &header;
    print "<P>Finished. The content has been written to $full_textfilename</P>\n";
    print "<P>To preview the page, click <a target=\"_blank\" href=\"$content_script?page=/$pagename\">here</a></P>\n";
    &footer;
}


sub show_help {
    # pop-up help in a separate browser window

    &header;

    print <<OUTPUT;
    <H3>Help for the user</H3>

    <P>Complete all the relevant fields on the main page. Note that you can upload images directly from your desktop to the server using the relevant button, and these can then be
    incorporated into your text using a link such as <B>&lt;IMG SRC=&quot;photo.jpg&quot; ALT=&quot;our boss&quot;&gt;</B>. You don't need to include a full path or image dimensions - the CMS will work those out for you.</P>
    <P>If you know the basic HTML formatting tags (for bold, underline, italic and bulleted lists), you can use these to emphasis your text.</P>
    <P>When linking to other pages within the site, use<BR><B>&lt;A HREF=&quot;/products.htm&quot;&gt;Our Products&lt;/A&gt;</B><BR>for example.</P>
    <P>When linking to other pages on other sites, use<BR><B>&lt;A TARGET=&quot;_BLANK&quot; HREF=&quot;http://www.other.co.uk/products.htm&quot;&gt;Other products&lt;/A&gt;</B><BR>for example.</P>

OUTPUT

    &footer;
}


sub header {
    # header for all pages

    print "<HTML><HEAD><TITLE>$title</TITLE></HEAD>\n";
    print "<BODY BGCOLOR=\"#E0E0FF\"><TABLE WIDTH=\"610\" CELLPADDING=\"2\" CELLSPACING=\"0\" BORDER=\"0\" ALIGN=\"CENTER\">\n";
    print "<TR><TD ALIGN=\"LEFT\" VALIGN=\"TOP\" BGCOLOR=\"#E0E0FF\" WIDTH=\"100%\">\n";
    print "<FONT FACE=\"arial, helvetica, sans-serif\" SIZE=\"2\" COLOR=\"#202090\">\n";
    print "<FONT COLOR=\"#0000FF\"><H2>$title</H2></FONT>\n";
}


sub footer {
    # footer for all pages

    print "</FONT></TD></TR></TABLE></BODY></HTML>\n";
}


sub init_security {
    # handles error reporting (for development purposes only), plus basic security

    # NEXT LINES ARE FOR TESTING, as is the Perl -w switch. REMOVE in online scripts
    use diagnostics;
    use diagnostics -verbose;
    enable diagnostics;
    use strict;
    use strict "refs";
    use strict "subs";
    no strict "vars";
    # ABOVE LINES ARE FOR TESTING

    # Security stuff
    use CGI;
    use CGI::Carp 'fatalsToBrowser';
    $CGI::POST_MAX=5*1024; # maximum of 5k length posts accepted - helps prevent DoS attacks
    $CGI::DISABLE_UPLOADS=1; # no uploads permitted - helps prevent DoS attacks
}


sub parse_form {
    # Form parsing, to handle data transmitted via POST commands

    unless ($ENV{'CONTENT_LENGTH'}) {return;}

    read(STDIN, $buffer, $ENV{'CONTENT_LENGTH'});
    @pairs = split(/&/, $buffer);
    foreach $pair (@pairs) {
        ($name, $value) = split(/=/, $pair);
        $value =~ tr/+/ /;
        $value =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
        if ($FORM{$name}) { $fields{$name} = $fields{$name}.",".$value; }
        else { $FORM{$name} = $value; }
    }
}


sub datecheck {
    # returns the day name (e.g Monday) if valid, or a zero if not
    # this sub-routine Copyright 2000, Jim Melanson, webmaster@thebeaches.to

    my $yr = $_[0]; my $mo = $_[1]; my $da = $_[2]; my $yr_length = length($yr);
    my %nm = (0 => 'Saturday', 1 => 'Sunday', 2 => 'Monday', 3 => 'Tuesday', 4 => 'Wednesday', 5 => 'Thursday', 6 => 'Friday');
    my %mc = (1 => [1, 31], 2 => [4, 28], 3 => [4, 31], 4 => [0, 30], 5 => [2, 31], 6 => [5, 30], 7 => [0, 31], 8 => [3, 31], 9 => [6, 30], 10 => [1, 31], 11 => [4, 30], 12 => [6, 31]);
    my %ly = ();
    if (($yr_length == 4) && ($yr >= 1800) && ($yr <= 2099) && ($da <= $mc{$mo}[1])) {
        for($a = 2000; $a >= 1640; $a--) {$ly{$a} = 1; $a -= 3}
        for($a = 2000; $a <= 4000; $a++) {$ly{$a} = 1; $a += 3}
        if($ly{$yr}) {$mc{'1'}[0] = 0; $mc{'2'}[0] = 3; $mc{'2'}[1]++}
        my $px = substr($yr, 0, 2); my $yc = substr($yr, 2, 2);
        my $yv = (int($yc / 4) + $yc + $mc{$mo}[0] + $da);
        my $rm = $yv - (int($yv / 7) * 7);
        if($px == 18) {$rm += 2}
        elsif($px == 20) {$rm--}
        $rm = 7 - abs($rm) if $rm < 0;
        return($nm{$rm});
        } else {
        return(0);
    }
}


sub process_html_tags () {
    # convert HTML tags into proprietary format.

    $_ = shift;

    # initial formatting
    s/(\s)?\n(\s)?\n(\s)?/\(\!paragraph\)/gis;
    s/(\s)?\n(\s)?/\(\!newline\)/gis;

    # find and convert external links
    s/\<a target=\"\_blank\" href=(\")?(.*?)(\")?\>(.*?)\<\/a\>/\(\!externallinkstart\=$2\)$4\(\!externallinkend\)/gis;
    s/\<a href=(\")?(.*?)(\")? target=\"\_blank\"\>(.*?)\<\/a\>/\(\!externallinkstart\=$2\)$4\(\!externallinkend\)/gis;
    s/\(\!externallinkstart\=http\:\/\/(.*?)\(\!externallinkend\)/\(\!externallinkstart\=$1\(\!externallinkend\)/gis;

    # next two lines deal with in-page links (i.e. <a href="#3"> and <a name="3">)
    s/\<a href=(\")?\#(.*?)(\")?\>(.*?)\<\/a\>/\(\!internalpagelinkstart\=$2\)$4\(\!internalpagelinkend\)/gis;
    s/\<a name=(\")?(.*?)(\")?\>(.*?)\<\/a\>/\(\!internalpagemarkerstart\=$2\)$4\(\!internalpagemarkerend\)/gis;

    # find and convert internal links
    s/\<a href=(\")?(.*?)(\")?\>(.*?)\<\/a\>/\(\!internallinkstart\=$2\)$4\(\!internallinkend\)/gis;

    # general formatting and tweaking
    s/\<FONT(.*?)SIZE=\"(.*?)\"\>(.*?)\<\/FONT\>/\(\!fontsize=$2\)$3\(\!fontsizeend\)/gis;
    s/\<FONT(.*?)COLOR=\"(.*?)\"\>(.*?)\<\/FONT\>/\(\!fontcolour=$2\)$3\(\!fontcolourend\)/gis;
    s/\<B\>(.*?)\<\/B\>/\(\!boldstart\)$1\(\!boldend\)/gis;
    s/\<I\>(.*?)\<\/I\>/\(\!italicstart\)$1\(\!italicend\)/gis;
    s/\<U\>(.*?)\<\/U\>/\(\!underlinestart\)$1\(\!underlineend\)/gis;
    s/\<UL\>\(\!newline\)\<LI\>/\<UL\>\<LI\>/gis;
    s/\<UL\>(.*?)\<\/UL\>/\(\!indentliststart\)$1\(\!indentlistend\)/gis;
    s/\<LI\>/\(\!bulletlistline\)/gis;
    s/\(\!newline\)(\s)?$//is;

    return $_;
}


sub unprocess_html_tags () {
    # convert proprietary tags back into HTML format - basically the opposite of the sub above

    $_ = shift;
    
    # handle the body image bit
    $body =~ s/\(\!bodyimagestart\)(.*?),(.*?)\(\!bodyimageend\)/<IMG SRC=\"$1\"$2\>/gis;
    $body =~ s/SRC\=\"$image_url/SRC\=\"/gi;

    # general formatting and tweaking
    s/\(\!bulletlistline\)/\<LI\>/gis;
    s/\(\!indentliststart\)(.*?)\(\!indentlistend\)/\<UL\>$1\<\/UL\>/gis;
    s/\<UL\>\<LI\>/\<UL\>\n\<LI\>/gis;
    s/\(\!underlinestart\)(.*?)\(\!underlineend\)/\<U\>$1\<\/U\>/gis;
    s/\(\!italicstart\)(.*?)\(\!italicend\)/\<I\>$1\<\/I\>/gis;
    s/\(\!boldstart\)(.*?)\(\!boldend\)/\<B\>$1\<\/B\>/gis;
    s/\(\!fontcolour=(.*?)\)(.*?)\(\!fontcolourend\)/\<FONT COLOR=\"$1\"\>$2\<\/FONT\>/gis;
    s/\(\!fontsize=(.*?)\)(.*?)\(\!fontsizeend\)/\<FONT SIZE=\"$1\"\>$2\<\/FONT\>/gis;

    # find and convert internal links
    s/\(\!internallinkstart\=(.*?)\)(.*?)\(\!internallinkend\)/\<a href=\"$1\"\>$2\<\/a\>/gis;

    # next two lines deal with in-page links (i.e. <a href="#3"> and <a name="3">)
    s/\(\!internalpagemarkerstart\=(.*?)\)(.*?)\(\!internalpagemarkerend\)/\<a name=\"$1\"\>$2\<\/a\>/gis;
    s/\(\!internalpagelinkstart\=(.*?)\)(.*?)\(\!internalpagelinkend\)/\<a href=\"$1\"\>$2\<\/a\>/gis;

    # find and convert external links
    s/\(\!externallinkstart\=(.*?)\)(.*?)\(\!externallinkend\)/\<a target=\"\_blank\" href=\"http\:\/\/$1\"\>$2\<\/a\>/gis;

    # initial formatting
    s/\(\!newline\)/\n/gis;
    s/\(\!paragraph\)/\n\n/gis;

    return $_;
}


sub dimension_body_images {
    # add dimensions of images to body pictures, using Image::Size PPM module

    $image_counter = ($body =~ s/<IMG SRC=(\")?(.*?)(\") (.*?)\>/\(\!bodyimagestart\)$2,$4\(\!bodyimageend\)/gis);
    $temp_image_counter=0;

    while ($temp_image_counter < $image_counter) {
        if ($body =~ m/\(\!bodyimagestart\)(.*?),(.*?)\(\!bodyimageend\)/i) {
            $temporary_startofimagetag = $1;
            $temporary_restofimagetag = $2;

            if (($temporary_restofimagetag =~ m/width/i)||($temporary_restofimagetag =~ m/height/i)) {
                $temporary_restofimagetag =~ s/width\=\"(.*?)\"(\s)?//ig;
                $temporary_restofimagetag =~ s/height\=\"(.*?)\"(\s)?//ig;
            }

            $fullpicturefilename = $image_path.$temporary_startofimagetag;
            ($picture_x, $picture_y) = imgsize($fullpicturefilename);
            unless ($picture_x) {
                $temporary_restofimagetag=">IMAGE ERROR HERE!<";
                $picture_x = 0;
                $picture_y = 0;
            }
            $fullpicturefilename =~ s/$image_path/$image_url/i;
            $body =~ s/\(\!bodyimagestart\)(.*?)\,(.*?)\(\!bodyimageend\)/\<IMG SRC=\"$fullpicturefilename\" WIDTH=\"$picture_x\" HEIGHT=\"$picture_y\" $temporary_restofimagetag\>/i;
        }
        $temp_image_counter++;
    }
    $body =~ s/<IMG SRC=(\")?(.*?)(\")(.*?)\>/\(\!bodyimagestart\)$2,$4\(\!bodyimageend\)/gis;
}

