#!/usr/bin/perl
# Change above so it points to perl. Next line controls output buffer.


########
#
# Content Management Script, content-serving section, (c) Alex Cruickshank 2002 (alex@itreviews.co.uk)
#
# This code is released under the GNU Public Licence (see www.gnu.org for details) and is supplied
# as is with no warranty either expressed or implied. No liability accepted for loss or damage of any kind.
#
# Always back up your data before using new software.
#
# You are free to use this program to manage and serve data on your site.
# Please notify the author of any such use, as well as bug reports, modifications, etc.
#
#
# file path to content text files, with trailing '/'
$content_path = "../data/";
#
# where to store any content-generation errors
$error_log = "../data/errors.log";
#
# e-mail address to which problems should be sent - must use \ before @
$webmaster_email = "me\@foo.com";
#
# path to HTML template file
$html_template = "../data/template.htm";
#
########


print "Content-type: text/html\n\n";

if (($ENV{'QUERY_STRING'} =~ m/page=\/(.*)/)||($ENV{'QUERY_STRING'} =~ m/page=\\(.*)/)) {$pagename="$1";}
else {$pagename="index";}

&prepare_page;
&serve_page;

exit;


sub prepare_page {

    # cautious of command line
    unless (($pagename =~ m/^(\w)+$/)||($pagename =~ m/^(\w)+\/(\w)+$/)) {
        &trap_error("Illegal filename");
    }
        
    # open the file
    $targetfile = $content_path . $pagename."\.txt";

    unless (-e $targetfile) {
        &trap_error("File not found - $targetfile");
    }

    open (READIN, "$targetfile") || &trap_error("$!");
    @rlines = <READIN>;
    close(READIN);

    # read in the variables
    foreach $readline (@rlines) {

        if ($readline =~ m/\heading\=(.*)/i) {
            chomp($html_title=$1);
            $maintitle=uc($html_title);
        }
        elsif ($readline =~ m/^pagename\=(.*)/i) {chomp($filepagename=$1);}
        elsif ($readline =~ m/^description\=(.*)/i) {chomp($description=$1);}
        elsif ($readline =~ m/^keywords\=(.*)/i) {chomp($keywords=$1);}
        elsif ($readline =~ m/^publication_date\=(.*)/i) {chomp($publication_date=$1);}
        elsif ($readline =~ m/^body\=(.*)/i) {chomp($body=$1);}
        elsif ($readline =~ m/^note\=(.*)/i) {chomp($note=$1);}
        else {next;}
    }

    # if filename and pagename don't match, or there's no title or body, generate an error
    unless (($filepagename eq $pagename)&&($html_title)&&($body)) {
        &trap_error("Incorrect file or file contents problem");
    }

    # tidy the content, recreating HTML tags
    $heading = &unprocess_html_tags($heading);
    $description = &unprocess_html_tags($description);
    $body = &unprocess_html_tags($body);
    $note = &unprocess_html_tags($note);

    $heading = &fix_special_chars($heading);
    $description = &fix_special_chars($description);
    $body = &fix_special_chars($body);
    $note = &fix_special_chars($note);

    # if any non-converted tags remain, handle error
    if (($heading =~ m/\(\!/)||($description =~ m/\(\!/)||($body =~ m/\(\!/)||($note =~ m/\(\!/)) {
        &trap_error("Page generation problem");
    }
}


sub serve_page {
    # handle the page generation

    open (TEMPLATE, $html_template);
    @template = <TEMPLATE>;
    close (TEMPLATE);
    foreach $template_line (@template) {
        
        if ($template_line =~ m/<\/TITLE>/i){ # insert the title and relevant meta tags
            $template_line =~ s/<\/TITLE>/ - $html_title<\/TITLE>/i;
            $template_line .= "<META NAME=\"keywords\" CONTENT=\"$keywords\">\n";
            $template_line .= "<META NAME=\"description\" CONTENT=\"$description\">\n";
        }
        
        elsif ($template_line =~ m/<!--body-->/i){ # insert body and associated components
            $template_line = "<H2>$maintitle</H2>\n";
            $template_line .= "<SMALL><I>$description ($publication_date)</I></SMALL><BR>\n";
            $template_line .= "$body\n";
            $template_line .= "<P><SMALL><I>$note</I></SMALL></P>\n";
        }
        
        print "$template_line";
    }
}


sub unprocess_html_tags () {
    # convert proprietary tags back into HTML format

    $_ = shift;
    
    # handle the body image bit
    $body =~ s/\(\!bodyimagestart\)(.*?),(.*?)\(\!bodyimageend\)/<IMG SRC=\"$1\"$2\>/gis;

    # general formatting and tweaking
    s/\(\!bulletlistline\)/\<LI\>/gis;
    s/\(\!indentliststart\)(.*?)\(\!indentlistend\)/\<UL\>$1\<\/UL\>/gis;
    s/\<UL\>\<LI\>/\<UL\>\n\<LI\>/gis;
    s/\(\!underlinestart\)(.*?)\(\!underlineend\)/\<U\>$1\<\/U\>/gis;
    s/\(\!italicstart\)(.*?)\(\!italicend\)/\<I\>$1\<\/I\>/gis;
    s/\(\!boldstart\)(.*?)\(\!boldend\)/\<B\>$1\<\/B\>/gis;
    s/\(\!fontcolour=(.*?)\)(.*?)\(\!fontcolourend\)/\<FONT COLOR=\"$1\"\>$2\<\/FONT\>/gis;
    s/\(\!fontsize=(.*?)\)(.*?)\(\!fontsizeend\)/\<FONT SIZE=\"$1\"\>$2\<\/FONT\>/gis;

    # find and convert internal links
    s/\(\!internallinkstart\=(.*?)\)(.*?)\(\!internallinkend\)/\<a href=\"$1\"\>$2\<\/a\>/gis;

    # next two lines deal with in-page links (i.e. <a href="#3"> and <a name="3">)
    s/\(\!internalpagemarkerstart\=(.*?)\)(.*?)\(\!internalpagemarkerend\)/\<a name=\"$1\"\>$2\<\/a\>/gis;
    s/\(\!internalpagelinkstart\=(.*?)\)(.*?)\(\!internalpagelinkend\)/\<a href=\"$1\"\>$2\<\/a\>/gis;

    # find and convert external links
    s/\(\!externallinkstart\=(.*?)\)(.*?)\(\!externallinkend\)/\<a target=\"\_blank\" href=\"http\:\/\/$1\"\>$2\<\/a\>/gis;

    # initial formatting
    s/\(\!newline\)/\n/gis;
    s/\(\!paragraph\)/\n\n/gis;

    return $_;
}


sub trap_error() {
    # handle errors

    ($error_message) = @_;
    unless ($error_message) {$error_message = "Unknown error";}

    $html_title = "Error!";
    $maintitle = "Error!";

    $body = "<P>Sorry, an error has occurred. Please let the <A HREF=\"mailto:$webmaster_email\">webmaster</A> know. Thank you.</P><P>Error: $error_message.</P>\n";
    &serve_page;

    # record the error
    $this_user = "$ENV{'REMOTE_HOST'}";
    unless ($this_user) {
        $this_user = "$ENV{'REMOTE_ADDR'}";
    }

    open (ERS, ">>$error_log");
    print ERS "$error_message, $ENV{'QUERY_STRING'}, $this_user\n";
    close (ERS);

    exit;
}


sub fix_special_chars {
    # replace 'illegal' characters with HTML code

    $_ = shift;

    s/\s\&\s/ \&amp; /g;

    s//\&iexcl;/g;
    s//\&brvbar;/g;
    s//\&uml;/g;

    s//\&macr;/g;
    s//\&acute;/g;
    s//\&cedil;/g;
    s//\&iquest;/g;
    s//\&plusmn;/g;
    s//\&laquo;/g;
    s//\&raquo;/g;
    s//\&times;/g;
    s//\&divide;/g;
    s//\&cent;/g;
    s//\&pound;/g;
    s//\&curren;/g;
    s//\&yen;/g;
    s//\&sect;/g;
    s//\&copy;/g;
    s//\&not;/g;
    s//\&reg;/g;
    s//\&deg;/g;
    s//\&micro;/g;
    s//\&para;/g;
    s//\&middot;/g;
    s//\&frac14;/g;
    s//\&frac12;/g;
    s//\&frac34;/g;
    s//\&sup1;/g;
    s//\&sup2;/g;
    s//\&sup3;/g;
    s//\&Aacute;/g;
    s//\&aacute;/g;
    s//\&Aacute;/g;
    s//\&aacute;/g;
    s//\&acirc;/g;
    s//\&Acirc;/g;
    s//\&acirc;/g;
    s//\&Agrave;/g;
    s//\&agrave;/g;
    s//\&Agrave;/g;
    s//\&agrave;/g;
    s//\&aring;/g;
    s//\&Aring;/g;
    s//\&aring;/g;
    s//\&Atilde;/g;
    s//\&atilde;/g;
    s//\&Atilde;/g;
    s//\&atilde;/g;
    s//\&Auml/g;
    s//\&auml;/g;
    s//\&Auml;/g;
    s//\&auml;/g;
    s//\&circ;/g;
    s//\&ordf;/g;
    s//\&ring;/g;
    s//\&AElig;/g;
    s//\&aelig;/g;
    s//\&AElig;/g;
    s//\&aelig;/g;
    s//\&Ccedil;/g;
    s//\&ccedil;/g;
    s//\&Ccedil;/g;
    s//\&ccedil;/g;
    s//\&ETH;/g;
    s//\&eth;/g;
    s//\&ieth;/g;
    s//\&Eacute;/g;
    s//\&eacute;/g;
    s//\&Eacute;/g;
    s//\&eacute;/g;
    s//\&Ecirc;/g;
    s//\&ecirc;/g;
    s//\&Ecirc;/g;
    s//\&ecirc;/g;
    s//\&Egrave;/g;
    s//\&egrave;/g;
    s//\&Egrave;/g;
    s//\&egrave;/g;
    s//\&Euml;/g;
    s//\&euml;/g;
    s//\&Euml;/g;
    s//\&euml;/g;
    s//\&Iacute;/g;
    s//\&iacute;/g;
    s//\&Iacute;/g;
    s//\&iacute;/g;
    s//\&Icirc;/g;
    s//\&icirc;/g;
    s//\&Icirc;/g;
    s//\&icirc;/g;
    s//\&Igrave;/g;
    s//\&igrave;/g;
    s//\&Igrave;/g;
    s//\&igrave;/g;
    s//\&Iuml;/g;
    s//\&iuml;/g;
    s//\&Iuml;/g;
    s//\&iuml;/g;
    s//\&Ntilde;/g;
    s//\&ntilde;/g;
    s//\&Ntilde;/g;
    s//\&ntilde;/g;
    s//\&Oacute;/g;
    s//\&oacute;/g;
    s//\&Oacute;/g;
    s//\&oacute;/g;
    s//\&Ocirc;/g;
    s//\&ocirc;/g;
    s//\&Ocirc;/g;
    s//\&ocirc;/g;
    s//\&Ograve;/g;
    s//\&ograve;/g;
    s//\&Ograve;/g;
    s//\&ograve;/g;
    s//\&ordm;/g;
    s//\&Oslash;/g;
    s//\&oslash;/g;
    s//\&Oslash;/g;
    s//\&oslash;/g;
    s//\&Otilde;/g;
    s//\&otilde;/g;
    s//\&Otilde;/g;
    s//\&otilde;/g;
    s//\&Ouml;/g;
    s//\&ouml;/g;
    s//\&Ouml;/g;
    s//\&ouml;/g;
    s//\&szlig;/g;
    s//\&THORN;/g;
    s//\&thorn;/g;
    s//\&THORN;/g;
    s//\&thorn;/g;
    s//\&Uacute;/g;
    s//\&uacute;/g;
    s//\&Uacute;/g;
    s//\&uacute;/g;
    s//\&Ucirc;/g;
    s//\&ucirc;/g;
    s//\&Ucirc;/g;
    s//\&ucirc;/g;
    s//\&Ugrave;/g;
    s//\&ugrave;/g;
    s//\&Ugrave;/g;
    s//\&ugrave;/g;
    s//\&Uuml;/g;
    s//\&uuml;/g;
    s//\&Uuml;/g;
    s//\&uuml;/g;
    s//\&Yacute;/g;
    s//\&yacute;/g;
    s//\&Yacute;/g;
    s//\&yacute;/g;
    s//\&yuml;/g;

    return $_;
}
